#Colifrapy

##Description
Colifrapy is a Command Line Framework for Python.
Its aim is to provide several tools to build robust and
structured command line tools very easily.

##Installation
It is recommanded to use colifrapy under a python virtualenv.
To install, just type :

```
pip install colifrapy
```

##Philosophy
The goal of colifrapy is to load a yaml configuration file managing 
several settings such as your command line arguments,
version name and, if needed, your contextual settings.

Once those settings are loaded, you can call upon colifrapy models to do the
work while remaining able to access critical utilities such as argv opts, settings 
and logger.

##Usage

###Scaffolding
Colifrapy is able to create a new blank project for you.

```sh
colifrapy new [project]
```

This will create the necessary files to start working immediatly. I.e. a command line hub, a base
controller, a config file and a string file.

To test if the scaffolding has worked :
```sh
cd [project]
python [project].py anything
```

It should output a header display your project's title.

###Settings
The Settings class is the first thing you will have to import in order to use Colifrapy.
It has to load your yaml settings file to perform its magic.

config/settings.yml file:

```yaml
# Basic Informations
version: 'Colifrapy 0.1'
description: 'Description of Colifrapy'
strings: 'colifrapy/resources/strings.example.yml'
arguments: 
- [ ['-t', '--test'], {'help' : 'Test', 'type' : 'int'} ]
- [ ['positionnal_argument'] ]

# Generic Settings needed by your program
settings:
    hello: 'world'
    bonjour: 'test'
    hash: {'test' : 2}
```

And this is how you setup colifrapy (equivalent of [project].py file generated by the scaffolder):

```python
# Importing
from colifrapy import Settings, Commander

# Loading settings
settings = Settings()
settings.load('path/to/your/settings.yml')
```
If you do not give a path to the loading function, colifrapy will assume that
the path of your settings file is 'config/settings.yml'.

After loading the yml file, you will be able to access your generic settings anywhere
in you code.

Example:
```python
from colifrapy import Settings

settings = Settings()
print settings.hello
>>> 'world'

print settings.hash['test']
>>> 2
```

###Command
Arguments are to be defined as for the python [ArgParser](http://docs.python.org/dev/library/argparse.html "ArgParser") class.

Examples of argument declaration:
```yaml
arguments: 
- [ ['-t', '--test'], {'help' : 'Test', 'type' : 'int', 'default' : 5} ]
- [ ['-b', '--blue'], {'help' : 'Blue option', 'type' : 'int', 'required' : 'True'} ]
- [ ['some_positionnal_argument'] ]
```

Once the settings are loaded, you can access your options through:
```python
from colifrapy import Commander
command = Commander()

print command.opts.test
>>> 5
```

###Model
Models are the bulk of Colifrapy. You can extend them to acces your settings and commands easily.

Example:
```python
from colifrapy import Model

class MyModel(Model):
	def test(self):
		print self.settings.hello

m = MyModel()
m.test()
>>> 'world'
```

Reserved attributes names are:

	log (access to the logger described right after)
	opts (access to the command line options)
	settings (access to the program's settings)


###Logger and strings
The logger is the outputting class of colifrapy. It should be loaded with some strings by the settings.
If no strings are given, the logger will just output normally the argument string you give it.

The logger accepts six levels :

	INFO (green output)
	VERBOSE (cyan output)
	DEBUG (blue output)
	WARNING (yellow ouput)
	ERROR (red output)
	FATAL (purple output) --> will throw an exception for you to catch or not

By default, if no level is specified for a message, DEBUG will always be taken.

The fact that strings are externalized permits you to quickly modify the strings if
needed ,or even translate them easily.

Strings given must follow this yaml layout:
```yaml
main:
    process:
    	# String with a variable contained within the brackets
        start: 'Starting corpus analysis (path : {path})//INFO'
        # Simply write two slashes at the end to specify the level of the message
        end: 'Exiting//ERROR'
        test_line_break: '\nBonjour'
    title: 'Colifrapy'
other_string_category:
	test: 'Hello everyone//INFO'
```
And this is how you use the logger:
```python
from colifrapy import Model

class MyModel(Model):
	def test(self):

		# Main method
		#------------

		# Outputting a message
		self.log.write('main:process:end')
		>>> [ERROR] :: Exiting

		# Overriding the message level
		self.log.write('main:process:end', 'INFO')
		>>> [INFO] :: Exiting

		# Passing variables
		self.log.write('main:protocol:start', variables={'path' : 'test'})
		>>> [INFO] :: Starting corpus analysis (path : test)

		# Wrong message if yml file specified
		self.log.write('main:wrong_message')
		>>> Exception Colifrapy::Logger::WrongMessage

		# When yml file is not specified
		self.log.write('Test string', 'DEBUG')
		>>> [DEBUG] :: Test string

		# Helper methods
		#---------------

		# Printing a header (yellow color)
		self.log.header('main:title')
		>>> Colifrapy
		>>> ---------

		# Write methods shorteners
		self.log.error(message, vars)
		self.log.warning(...)
		self.log.info(...)
		self.log.debug(...)
		self.log.verbose(...)
		self.log.fatal(...)
```

Additionally, the logger write to a file which is located by default in 'logs/log.txt'.

You can set this path in your settings file along with the threshold of the logger and the
path to the strings.

Example:
```yaml
# Logger settings
string: 'path/to/your/strings.yml'
log_path: 'path/where/to/log/' # set to False, log won't be written to file
log_threshold : ['DEBUG', 'ERROR']
```

###Bonus
Colifrapy also gives access to a color class and a basic singleton decorator if needed.

```python
from colifrapy.tools.colors import color
print color('hello', 'blue')

from colifrapy.tools.decorators import singleton

@singleton
class MySingleton():
	pass
```

##Examples
My project [furuikeya](https://github.com/Yomguithereal/furuikeya) is a good example of the usage
of colifrapy since colifrapy was originally designed to help me doing it.

##Dependancies

	pyyaml
	pystache

##License
Colifrapy is under a MIT license.